package org.processmining.divideandconquer.extensions;

import java.net.URI;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.XGlobalAttributeNameMap;
import org.deckfour.xes.model.XAttributable;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

public class DrFurbyExtension extends XExtension {

	public static final URI EXTENSION_URI = URI.create("drfurby.xesext");
	
	public static final String KEY_NAME = "drfurby:name";
	public static final String KEY_POS = "drfurby:positive";
	public static final String KEY_NEG = "drfurby:negative";
	public static final String KEY_MILLIS = "drfurby:millis";
	
	public static final String KEY_CLASS = "drfurby:classification";
	public static final String KEY_TOTAL = "drfurby:totalCosts";
	public static final String KEY_IM100 = "drfurby:im100Costs";
	public static final String KEY_HIM0 = "drfurby:him0Costs";

	public static XAttributeLiteral ATTR_NAME;
	public static XAttributeDiscrete ATTR_POS;
	public static XAttributeDiscrete ATTR_NEG;
	public static XAttributeDiscrete ATTR_MILLIS;
	
	public static XAttributeLiteral ATTR_CLASS;
	public static XAttributeContinuous ATTR_TOTAL;
	public static XAttributeContinuous ATTR_IM100;
	public static XAttributeContinuous ATTR_HIM0;

	private transient static DrFurbyExtension singleton = new DrFurbyExtension();


	public static DrFurbyExtension instance() {
		return singleton;
	}

	private Object readResolve() {
		return singleton;
	}


	private DrFurbyExtension() {
		super("DrFurby", "drfurby", EXTENSION_URI);
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		
		ATTR_NAME = factory.createAttributeLiteral(KEY_NAME, "", this);
		ATTR_POS = factory.createAttributeDiscrete(KEY_POS, 0, this);
		ATTR_NEG = factory.createAttributeDiscrete(KEY_NEG, 0, this);
		ATTR_MILLIS = factory.createAttributeDiscrete(KEY_MILLIS, 0, this);
		
		ATTR_CLASS = factory.createAttributeLiteral(KEY_CLASS, "", this);
		ATTR_TOTAL = factory.createAttributeContinuous(KEY_TOTAL, 0.0, this);
		ATTR_IM100 = factory.createAttributeContinuous(KEY_IM100, 0.0, this);
		ATTR_HIM0 = factory.createAttributeContinuous(KEY_HIM0, 0.0, this);
		
		this.logAttributes.add((XAttribute) ATTR_NAME.clone());
		this.logAttributes.add((XAttribute) ATTR_POS.clone());
		this.logAttributes.add((XAttribute) ATTR_NEG.clone());
		this.logAttributes.add((XAttribute) ATTR_MILLIS.clone());
		
		this.traceAttributes.add((XAttribute) ATTR_CLASS.clone());
		this.traceAttributes.add((XAttribute) ATTR_TOTAL.clone());
		this.traceAttributes.add((XAttribute) ATTR_IM100.clone());
		this.traceAttributes.add((XAttribute) ATTR_HIM0.clone());

		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_NAME,
				"Name of Log used To Classify");
		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_POS,
				"Number of Traces Classified Positive");
		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_NEG,
				"Number of Traces Clasisfied Negative");
		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_MILLIS,
				"Time in Milliseconds Required for Classification");
		
		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_CLASS,
				"Classification (+/-)");
		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_TOTAL,
				"Accumulated Replay Costs");
		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_IM100,
				"IM100 Replay Costs");
		XGlobalAttributeNameMap.instance().registerMapping(
				XGlobalAttributeNameMap.MAPPING_ENGLISH, KEY_HIM0,
				"HIM0 Replay Costs");
	}

	public String extractName(XLog log) {
		return extractNamePrivate(log);
	}

	private String extractNamePrivate(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_NAME);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeLiteral) attribute).getValue();
		}
	}

	public String extractClass(XTrace trace) {
		return extractClassPrivate(trace);
	}

	private String extractClassPrivate(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_CLASS);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeLiteral) attribute).getValue();
		}
	}

	public Double extractTotal(XTrace trace) {
		return extractTotalPrivate(trace);
	}

	private Double extractTotalPrivate(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_TOTAL);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeContinuous) attribute).getValue();
		}
	}

	public Double extractIM100(XTrace trace) {
		return extractIM100Private(trace);
	}

	private Double extractIM100Private(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_IM100);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeContinuous) attribute).getValue();
		}
	}

	public Double extractHIM0(XTrace trace) {
		return extractHIM0Private(trace);
	}

	private Double extractHIM0Private(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_HIM0);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeContinuous) attribute).getValue();
		}
	}

	public Long extractMillis(XLog log) {
		return extractMillisPrivate(log);
	}

	private Long extractMillisPrivate(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_MILLIS);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeDiscrete) attribute).getValue();
		}
	}

	public Long extractPos(XLog log) {
		return extractPosPrivate(log);
	}

	private Long extractPosPrivate(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_POS);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeDiscrete) attribute).getValue();
		}
	}

	public Long extractNeg(XLog log) {
		return extractNegPrivate(log);
	}

	private Long extractNegPrivate(XAttributable element) {
		XAttribute attribute = element.getAttributes().get(KEY_NEG);
		if (attribute == null) {
			return null;
		} else {
			return ((XAttributeDiscrete) attribute).getValue();
		}
	}

	public void assignName(XLog log, String name) {
		assignNamePrivate(log, name);
	}

	private void assignNamePrivate(XAttributable element, String name) {
		if (name != null) {
			XAttributeLiteral attr = (XAttributeLiteral) ATTR_NAME
					.clone();
			attr.setValue(name);
			element.getAttributes().put(KEY_NAME, attr);
		}
	}

	public void assignClass(XTrace trace, String classification) {
		assignClassPrivate(trace, classification);
	}

	private void assignClassPrivate(XAttributable element, String classification) {
		if (classification != null) {
			XAttributeLiteral attr = (XAttributeLiteral) ATTR_CLASS
					.clone();
			attr.setValue(classification);
			element.getAttributes().put(KEY_CLASS, attr);
		}
	}

	public void assignTotal(XTrace trace, Double total) {
		assignTotalPrivate(trace, total);
	}

	private void assignTotalPrivate(XAttributable element, Double total) {
		if (total != null && total >= 0.0) {
			XAttributeContinuous attr = (XAttributeContinuous) ATTR_TOTAL
					.clone();
			attr.setValue(total);
			element.getAttributes().put(KEY_TOTAL, attr);
		}
	}

	public void assignIM100(XTrace trace, Double total) {
		assignIM100Private(trace, total);
	}

	private void assignIM100Private(XAttributable element, Double total) {
		if (total != null && total >= 0.0) {
			XAttributeContinuous attr = (XAttributeContinuous) ATTR_IM100
					.clone();
			attr.setValue(total);
			element.getAttributes().put(KEY_IM100, attr);
		}
	}

	public void assignHIM0(XTrace trace, Double total) {
		assignHIM0Private(trace, total);
	}

	private void assignHIM0Private(XAttributable element, Double total) {
		if (total != null && total >= 0.0) {
			XAttributeContinuous attr = (XAttributeContinuous) ATTR_HIM0
					.clone();
			attr.setValue(total);
			element.getAttributes().put(KEY_HIM0, attr);
		}
	}

	public void assignMillis(XLog log, Long millis) {
		assignMillisPrivate(log, millis);
	}

	private void assignMillisPrivate(XAttributable element, Long millis) {
		if (millis != null && millis >= 0) {
			XAttributeDiscrete attr = (XAttributeDiscrete) ATTR_MILLIS
					.clone();
			attr.setValue(millis);
			element.getAttributes().put(KEY_MILLIS, attr);
		}
	}

	public void assignPos(XLog log, Long pos) {
		assignPosPrivate(log, pos);
	}

	private void assignPosPrivate(XAttributable element, Long pos) {
		if (pos != null && pos >= 0) {
			XAttributeDiscrete attr = (XAttributeDiscrete) ATTR_POS
					.clone();
			attr.setValue(pos);
			element.getAttributes().put(KEY_POS, attr);
		}
	}

	public void assignNeg(XLog log, Long millis) {
		assignNegPrivate(log, millis);
	}

	private void assignNegPrivate(XAttributable element, Long neg) {
		if (neg != null && neg >= 0) {
			XAttributeDiscrete attr = (XAttributeDiscrete) ATTR_NEG
					.clone();
			attr.setValue(neg);
			element.getAttributes().put(KEY_NEG, attr);
		}
	}

}
