package org.processmining.plugins;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.classifiers.StartEndClassifier;
import org.processmining.connections.RemoveStartEndTransitionsConnection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.AcceptingPetriNet;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.impl.DivideAndConquerFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.parameters.RemoveStartEndTransitionsParameters;
import org.processmining.utils.PetriNetUtils;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.acceptingpetrinet.plugins.RemoveStartEndTransitionsPlugin instead.
 */
@Deprecated
//Plugin(name = "Remove Start and End Transitions", parameterLabels = { "Accepting Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class })
public class RemoveStartEndTransitionsPlugin {

//	UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
//	PluginVariant(variantLabel = "Remove Start and End Transitions, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet reduceDefault(PluginContext context, AcceptingPetriNet net) {
		RemoveStartEndTransitionsParameters parameters = new RemoveStartEndTransitionsParameters();
		return reducePrivateConnection(context, net, parameters);
	}

//	PluginVariant(variantLabel = "Remove Start and End Transitions, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet reduceParameters(PluginContext context, AcceptingPetriNet net,
			RemoveStartEndTransitionsParameters parameters) {
		return reducePrivateConnection(context, net, parameters);
	}

	private AcceptingPetriNet reducePrivateConnection(PluginContext context, AcceptingPetriNet net,
			RemoveStartEndTransitionsParameters parameters) {
		Collection<RemoveStartEndTransitionsConnection> connections;
		try {
			connections = context.getConnectionManager().getConnections(RemoveStartEndTransitionsConnection.class,
					context, net);
			for (RemoveStartEndTransitionsConnection connection : connections) {
				if (connection.getObjectWithRole(RemoveStartEndTransitionsConnection.NET).equals(net)
						&& connection.getParameters().equals(parameters)) {
					return connection.getObjectWithRole(RemoveStartEndTransitionsConnection.REDUCEDNET);
				}
			}
		} catch (ConnectionCannotBeObtained e) {
		}
		AcceptingPetriNet reducedNet = reducePrivate(context, net, parameters);
		context.getProvidedObjectManager().createProvidedObject(
				"Initial marking for " + reducedNet.getNet().getLabel(), reducedNet.getInitialMarking(), Marking.class,
				context);
		context.addConnection(new InitialMarkingConnection(reducedNet.getNet(), reducedNet.getInitialMarking()));
		context.getConnectionManager().addConnection(
				new RemoveStartEndTransitionsConnection(net, reducedNet, parameters));
		return reducedNet;
	}

	private AcceptingPetriNet reducePrivate(PluginContext context, AcceptingPetriNet net,
			RemoveStartEndTransitionsParameters parameters) {
		AcceptingPetriNet reducedNet = DivideAndConquerFactory.createAcceptingPetriNet();

		Map<Place, Place> placeMap = new HashMap<Place, Place>();
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>();
		Petrinet pn = PetriNetUtils.cloneNet(net.getNet(), placeMap, transitionMap);
		Marking initialMarking = PetriNetUtils.apply(net.getInitialMarking(), placeMap);
		Set<Marking> finalMarkings = PetriNetUtils.apply(net.getFinalMarkings(), placeMap);

		Set<Transition> transitions = new HashSet<Transition>(pn.getTransitions());
		for (Transition transition : transitions) {
			if (transition.getLabel().equals(StartEndClassifier.START)) {
				if (parameters.getAbstraction().equals(RemoveStartEndTransitionsParameters.HIDE)) {
					transition.setInvisible(true);
				} else if (parameters.getAbstraction().equals(RemoveStartEndTransitionsParameters.REMOVE)) {
					int minTokens = Integer.MAX_VALUE;
					boolean ok = true;
					/*
					 * Check whether all input places have no inputs and only
					 * this transition as output.
					 */
					for (PetrinetEdge<?, ?> edge : pn.getInEdges(transition)) {
						if (edge instanceof Arc) {
							Arc arc = (Arc) edge;
							Place place = (Place) arc.getSource();
							ok = ok && pn.getInEdges(place).isEmpty() && pn.getOutEdges(place).size() == 1;
						}
					}
					if (ok) {
						for (PetrinetEdge<?, ?> edge : pn.getInEdges(transition)) {
							if (edge instanceof Arc) {
								Arc arc = (Arc) edge;
								Place place = (Place) arc.getSource();
								if (initialMarking.occurrences(place) < minTokens) {
									minTokens = initialMarking.occurrences(place);
								}
								pn.removePlace(place);
								while (initialMarking.contains(place)) {
									initialMarking.remove(place);
								}
							}
						}
						for (PetrinetEdge<?, ?> edge : pn.getOutEdges(transition)) {
							if (edge instanceof Arc) {
								Arc arc = (Arc) edge;
								Place place = (Place) arc.getTarget();
								initialMarking.add(place, minTokens);
							}
						}
						pn.removeTransition(transition);
					} else {
						transition.setInvisible(true);
					}
				}
			} else if (transition.getLabel().equals(StartEndClassifier.END)) {
				if (parameters.getAbstraction().equals(RemoveStartEndTransitionsParameters.HIDE)) {
					transition.setInvisible(true);
				} else if (parameters.getAbstraction().equals(RemoveStartEndTransitionsParameters.REMOVE)) {
					int maxTokens = 1;
					boolean ok = true;
					for (PetrinetEdge<?, ?> edge : pn.getOutEdges(transition)) {
						if (edge instanceof Arc) {
							Arc arc = (Arc) edge;
							Place place = (Place) arc.getTarget();
							ok = ok && pn.getOutEdges(place).isEmpty() && pn.getInEdges(place).size() == 1;
						}
					}
					if (ok) {
						for (PetrinetEdge<?, ?> edge : pn.getOutEdges(transition)) {
							if (edge instanceof Arc) {
								Arc arc = (Arc) edge;
								Place place = (Place) arc.getTarget();
								if (initialMarking.occurrences(place) > maxTokens) {
									maxTokens = initialMarking.occurrences(place);
								}
								pn.removePlace(place);
							}
						}
						Marking finalMarking = new Marking();
						for (PetrinetEdge<?, ?> edge : pn.getInEdges(transition)) {
							if (edge instanceof Arc) {
								Arc arc = (Arc) edge;
								Place place = (Place) arc.getSource();
								finalMarking.add(place, maxTokens);
							}
						}
						// Set the only final marking to the minimal marking that enables this transition.
						finalMarkings.clear();
						finalMarkings.add(finalMarking);
						// Remove this transition.
						pn.removeTransition(transition);
					} else {
						transition.setInvisible(true);
					}
				}
			}
		}
		reducedNet.init(pn);
		reducedNet.setInitialMarking(initialMarking);
		reducedNet.setFinalMarkings(finalMarkings);
		return reducedNet;
	}

}
