package org.processmining.plugins;

import java.util.Collection;

import org.processmining.connections.RemoveStructuralRedundantPlacesConnection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.AcceptingPetriNet;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.impl.DivideAndConquerFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.parameters.BerthelotParameters;
import org.processmining.parameters.RemoveStructuralRedundantPlacesParameters;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.acceptingpetrinetredundantplacesreductor.plugins.RemoveStructuralRedundantPlacesPlugin instead.
 */
@Deprecated
//Plugin(name = "Remove Structural Redundant Places from Accepting Petri Net", parameterLabels = {
//		"Accepting Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class })
public class RemoveStructuralRedundantPlacesPlugin {

//	UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
//	PluginVariant(variantLabel = "Remove Structural Redundant Places, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet reduceDefault(PluginContext context, AcceptingPetriNet net) {
		RemoveStructuralRedundantPlacesParameters parameters = new RemoveStructuralRedundantPlacesParameters();
		return reducePrivateConnection(context, net, parameters);
	}

//	PluginVariant(variantLabel = "Remove Structural Redundant Places, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet reduceParameters(PluginContext context, AcceptingPetriNet net,
			RemoveStructuralRedundantPlacesParameters parameters) {
		return reducePrivateConnection(context, net, parameters);
	}

	private AcceptingPetriNet reducePrivateConnection(PluginContext context, AcceptingPetriNet net,
			RemoveStructuralRedundantPlacesParameters parameters) {
		Collection<RemoveStructuralRedundantPlacesConnection> connections;
		if (context != null) {
			try {
				connections = context.getConnectionManager().getConnections(
						RemoveStructuralRedundantPlacesConnection.class, context, net);
				for (RemoveStructuralRedundantPlacesConnection connection : connections) {
					if (connection.getObjectWithRole(RemoveStructuralRedundantPlacesConnection.NET).equals(net)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(RemoveStructuralRedundantPlacesConnection.REDUCEDNET);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNet reducedNet = reducePrivate(context, net, parameters);
		if (context != null) {
			context.getProvidedObjectManager().createProvidedObject(
					"Initial marking for " + reducedNet.getNet().getLabel(), reducedNet.getInitialMarking(),
					Marking.class, context);
			context.addConnection(new InitialMarkingConnection(reducedNet.getNet(), reducedNet.getInitialMarking()));
			context.getConnectionManager().addConnection(
					new RemoveStructuralRedundantPlacesConnection(net, reducedNet, parameters));
		}
		return reducedNet;
	}

	private AcceptingPetriNet reducePrivate(PluginContext context, AcceptingPetriNet net,
			RemoveStructuralRedundantPlacesParameters parameters) {
		AcceptingPetriNet reducedNet = DivideAndConquerFactory.createAcceptingPetriNet();
		BerthelotParameters berthelotParameters = new BerthelotParameters();
		berthelotParameters.setInitialMarking(net.getInitialMarking());
		berthelotParameters.setFinalMarkings(net.getFinalMarkings());
		BerthelotPlugin berthelotPlugin = new BerthelotPlugin();
		Petrinet pn = berthelotPlugin.reduceParameters(null, net.getNet(), berthelotParameters);
		reducedNet.init(pn);
		reducedNet.setInitialMarking(berthelotParameters.getInitialBerthelotMarking());
		reducedNet.setFinalMarkings(berthelotParameters.getFinalBerthelotMarkings());
		return reducedNet;
	}

}