package org.processmining.plugins;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.processmining.connections.UnpackPetrinetConnection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.AcceptingPetriNet;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.parameters.UnpackPetrinetParameters;
import org.processmining.utils.PetriNetUtils;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.acceptingpetrinet.plugins.UnpackPetrinetPlugin instead.
 */
@Deprecated
//Plugin(name = "Unpack Accepting Petri net", parameterLabels = { "Accepting Petri Net", "Parameters" }, returnLabels = { "Petri Net" }, returnTypes = { Petrinet.class })
public class UnpackPetrinetPlugin {

//	UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
//	PluginVariant(variantLabel = "Unpack Accepting Petri net, Default", requiredParameterLabels = { 0 })
	public Petrinet unpackDefault(PluginContext context, AcceptingPetriNet apn) {
		UnpackPetrinetParameters parameters = new UnpackPetrinetParameters();
		return unpackPrivateConnection(context, apn, parameters);
	}

//	PluginVariant(variantLabel = "Unpack Accepting Petri net, Parameters", requiredParameterLabels = { 0, 1 })
	public Petrinet reduceParameters(PluginContext context, AcceptingPetriNet apn, UnpackPetrinetParameters parameters) {
		return unpackPrivateConnection(context, apn, parameters);
	}

	private Petrinet unpackPrivateConnection(PluginContext context, AcceptingPetriNet apn,
			UnpackPetrinetParameters parameters) {
		Collection<UnpackPetrinetConnection> connections;
		try {
			connections = context.getConnectionManager().getConnections(UnpackPetrinetConnection.class, context, apn);
			for (UnpackPetrinetConnection connection : connections) {
				if (connection.getObjectWithRole(UnpackPetrinetConnection.APN).equals(apn)
						&& connection.getParameters().equals(parameters)) {
					return connection.getObjectWithRole(UnpackPetrinetConnection.NET);
				}
			}
		} catch (ConnectionCannotBeObtained e) {
		}
		Petrinet net = unpackPrivate(context, apn, parameters);
		context.getConnectionManager().addConnection(new UnpackPetrinetConnection(apn, net, parameters));
		return net;
	}

	private Petrinet unpackPrivate(PluginContext context, AcceptingPetriNet apn, UnpackPetrinetParameters parameters) {
		Map<Place, Place> placeMap = new HashMap<Place, Place>(apn.getNet().getPlaces().size());
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>(apn.getNet().getTransitions()
				.size());
		Petrinet net = PetriNetUtils.cloneNet(apn.getNet(), placeMap, transitionMap);
		Marking initialMarking = PetriNetUtils.apply(apn.getInitialMarking(), placeMap);
		Set<Marking> finalMarkings = PetriNetUtils.apply(apn.getFinalMarkings(), placeMap);
		//		context.getProvidedObjectManager().createProvidedObject("Petri net " + net.getLabel(), net, Petrinet.class, context);

		context.getProvidedObjectManager().createProvidedObject("Initial marking for " + net.getLabel(),
				initialMarking, Marking.class, context);
		context.addConnection(new InitialMarkingConnection(net, initialMarking));

		for (Marking finalMarking : finalMarkings) {
			context.getProvidedObjectManager().createProvidedObject("Final marking for " + net.getLabel(),
					finalMarking, Marking.class, context);
			context.addConnection(new FinalMarkingConnection(net, finalMarking));
		}

		return net;
	}
}
