package org.processmining.plugins;

import java.awt.Color;
import java.awt.Dimension;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.models.CausalActivityGraph;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.AttributeMap.ArrowType;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphEdge;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.jgraph.ProMJGraphVisualizer;
import org.processmining.models.shapes.RoundedRect;

/**
 * 
 * @author hverbeek
 * @deprecated Please use org.processmining.causalactivitygraph.plugins.VisualizeCausalActivityGraph instead.
 */
@Deprecated
public class VisualizeCausalActivityGraph {

//	Plugin(name = "Visualize Causal Activity Graph", returnLabels = { "Visualized Causal Activity Graph" }, returnTypes = { JComponent.class }, parameterLabels = { "Causal Activity Matrix" }, userAccessible = false)
//	Visualizer
//	PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(UIPluginContext context, CausalActivityGraph graph) {
		VisualizedGraph view = new VisualizedGraph(context, graph);
		return ProMJGraphVisualizer.instance().visualizeGraph(context, view);
	}

	private class VisualizedNode extends AbstractDirectedGraphNode {

		VisualizedGraph view;

		public VisualizedNode(VisualizedGraph view, XEventClass activity) {
			this.view = view;
			getAttributeMap().put(AttributeMap.SHAPE, new RoundedRect());
			getAttributeMap().put(AttributeMap.SQUAREBB, false);
			getAttributeMap().put(AttributeMap.RESIZABLE, false);
			getAttributeMap().put(AttributeMap.SIZE, new Dimension(100, 60));
			getAttributeMap().put(AttributeMap.FILLCOLOR, Color.LIGHT_GRAY);
			getAttributeMap().put(AttributeMap.LABEL, activity.getId());
		}

		public AbstractDirectedGraph<?, ?> getGraph() {
			return view;
		}

	}

	private class VisualizedEdge extends AbstractDirectedGraphEdge<VisualizedNode, VisualizedNode> {

		public VisualizedEdge(VisualizedNode source, VisualizedNode target, double weight) {
			super(source, target);
			getAttributeMap().put(AttributeMap.EDGEEND, ArrowType.ARROWTYPE_CLASSIC);
			getAttributeMap().put(AttributeMap.EDGEENDFILLED, true);
			getAttributeMap().put(AttributeMap.LABEL, "" + weight);
			int rgbValue = (int) ((1 - weight) * 255);
			getAttributeMap().put(AttributeMap.EDGECOLOR, new Color(rgbValue, rgbValue, 0));
//			getAttributeMap().put(AttributeMap.LINEWIDTH, (float) (10 * weight + 1));
		}

	}

	private class VisualizedGraph extends AbstractDirectedGraph<VisualizedNode, VisualizedEdge> {

		private CausalActivityGraph graph;
		private Map<XEventClass, VisualizedNode> nodeMap = null;
		private Map<Pair<XEventClass, XEventClass>, VisualizedEdge> edgeMap = null;

		public VisualizedGraph(PluginContext context, CausalActivityGraph graph) {
			this.graph = graph;
		}

		public void init() {
			if (nodeMap == null && edgeMap == null) {
				nodeMap = new HashMap<XEventClass, VisualizedNode>();
				for (XEventClass activity : graph.getSetActivities()) {
					nodeMap.put(activity, new VisualizedNode(this, activity));
				}
				edgeMap = new HashMap<Pair<XEventClass, XEventClass>, VisualizedEdge>();
				for (Pair<XEventClass, XEventClass> causality : graph.getSetCausalities()) {
					edgeMap.put(causality,
							new VisualizedEdge(nodeMap.get(causality.getFirst()), nodeMap.get(causality.getSecond()),
									graph.getCausality(causality.getFirst(), causality.getSecond())));
				}
			}
		}

		public Set<VisualizedNode> getNodes() {
			init();
			return new HashSet<VisualizedNode>(nodeMap.values());
		}

		public Set<VisualizedEdge> getEdges() {
			init();
			return new HashSet<VisualizedEdge>(edgeMap.values());
		}

		public void removeNode(DirectedGraphNode cell) {
		}

		protected AbstractDirectedGraph<VisualizedNode, VisualizedEdge> getEmptyClone() {
			return new VisualizedGraph(null, graph);
		}

		protected Map<DirectedGraphElement, DirectedGraphElement> cloneFrom(
				DirectedGraph<VisualizedNode, VisualizedEdge> graph) {
			VisualizedGraph clone = new VisualizedGraph(null, this.graph);
			clone.init();
			Map<DirectedGraphElement, DirectedGraphElement> map = new HashMap<DirectedGraphElement, DirectedGraphElement>();
			for (XEventClass activity : nodeMap.keySet()) {
				map.put(this.nodeMap.get(activity), clone.nodeMap.get(activity));
			}
			return map;
		}

		public void removeEdge(DirectedGraphEdge edge) {
		}

	}
}
